@@ -1,5 +1,4 @@
 import copy
-import random
 import sys
 
 from datetime import datetime
@@ -315,8 +314,7 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         **scope_args  # type: Dict[str, Any]
     ):
         # type: (...) -> Optional[str]
-        """Captures an event. Alias of :py:meth:`sentry_sdk.Client.capture_event`.
-        """
+        """Captures an event. Alias of :py:meth:`sentry_sdk.Client.capture_event`."""
         client, top_scope = self._stack[-1]
         scope = _update_scope(top_scope, scope, scope_args)
         if client is not None:
@@ -506,20 +504,28 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         When the transaction is finished, it will be sent to Sentry with all its
         finished child spans.
         """
+        custom_sampling_context = kwargs.pop("custom_sampling_context", {})
+
+        # if we haven't been given a transaction, make one
         if transaction is None:
             kwargs.setdefault("hub", self)
             transaction = Transaction(**kwargs)
 
-        client, scope = self._stack[-1]
-
-        if transaction.sampled is None:
-            sample_rate = client and client.options["traces_sample_rate"] or 0
-            transaction.sampled = random.random() < sample_rate
+        # use traces_sample_rate, traces_sampler, and/or inheritance to make a
+        # sampling decision
+        sampling_context = {
+            "transaction_context": transaction.to_json(),
+            "parent_sampled": transaction.parent_sampled,
+        }
+        sampling_context.update(custom_sampling_context)
+        transaction._set_initial_sampling_decision(sampling_context=sampling_context)
 
+        # we don't bother to keep spans if we already know we're not going to
+        # send the transaction
         if transaction.sampled:
             max_spans = (
-                client and client.options["_experiments"].get("max_spans") or 1000
-            )
+                self.client and self.client.options["_experiments"].get("max_spans")
+            ) or 1000
             transaction.init_span_recorder(maxlen=max_spans)
 
         return transaction
@@ -685,12 +691,7 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         if not propagate_traces:
             return
 
-        if client and client.options["traceparent_v2"]:
-            traceparent = span.to_traceparent()
-        else:
-            traceparent = span.to_legacy_traceparent()
-
-        yield "sentry-trace", traceparent
+        yield "sentry-trace", span.to_traceparent()
 
 
 GLOBAL_HUB = Hub()
